using System;
using System.IO;
using System.Reflection;
using System.Threading;
using Ellisys.Platform.NetworkRemoteControl.Analyzer;
using Ellisys.Platform.NetworkRemoteControl.Analyzer.Ethernet;

namespace RemoteControl_Analyzer_Sample;

class Program
{
	// TODO: Please adapt the following constants to your needs
	private const string HostIp = "localhost";
	private const int HostPort = 12345;

	private static void Main(string[] args)
	{
		using var ice = Ice.Util.initialize(ref args);

		//
		// Creating proxy to the analyzer application
		//
		var btRemoteControl = CreateBluetoothRemoteControlProxy(ice);

		Console.WriteLine("Remote control connection succeeded.");
		Console.WriteLine();

		//
		// Loading sample trace
		//
		const string SampleTrace = "A2dpMusicStreaming.btt";
		const string SampleOverview = "BR/EDR";

		Console.WriteLine($"Open sample trace '{SampleTrace}'");
		Console.WriteLine();

		var tracePath = Path.Combine(Path.GetDirectoryName(Assembly.GetEntryAssembly().Location), SampleTrace);

		btRemoteControl.StartLoading(tracePath);

		//
		// Wait trace loaded
		//
		while (btRemoteControl.IsLoading())
		{
			Thread.Sleep(1000);
		}

		//
		// Printing some protocol items
		//
		btRemoteControl.SelectOverview(SampleOverview);
		btRemoteControl.SelectProtocolLayer("All Layers");

		var root = btRemoteControl.OverviewRootItem();

		const int TopItemsCount = 3;
		const int MaxDepth = 2;

		Console.WriteLine($"Print the first {TopItemsCount} top items with a max depth of {MaxDepth}");
		Console.WriteLine();

		PrintItems(btRemoteControl, root, TopItemsCount, MaxDepth, 0);
		Console.WriteLine();

		//
		// Printing channels statistics
		//
		Console.WriteLine("Print channels statistics");
		Console.WriteLine();

		PrintChannelsStatistics(btRemoteControl);

		//
		// Adding a WiFi key
		//
		var wifiRemoteControl = CreateEthernetRemoteControlProxy(ice);

		wifiRemoteControl.AddWifiKeyByApSsid("testSsid", "testKey");
	}

	private static void PrintItems(AnalyzerRemoteControlPrx remoteControl, int itemHandle, int printCount, int maxDepth, int currentDepth)
	{
		if (currentDepth >= maxDepth)
		{
			return;
		}

		var childCount = remoteControl.GetOverviewItemChildCount(itemHandle);

		for (var childIndex = 0; childIndex < Math.Min(childCount, printCount); childIndex++)
		{
			var childHandle = remoteControl.GetOverviewItemChild(itemHandle, childIndex);
			var childItemDescription = remoteControl.GetOverviewItemDescription(childHandle);

			Console.Write("".PadLeft(4 * currentDepth));
			Console.WriteLine($"{childItemDescription}");

			PrintItems(remoteControl, childHandle, 16, maxDepth, currentDepth + 1);
		}
	}

	private static void PrintChannelsStatistics(BluetoothAnalyzerRemoteControlPrx remoteControl)
	{
		var stats = remoteControl.GetChannelsSummary(0, long.MaxValue);

		Console.WriteLine("{0,8} {1,8} {2,8} {3,8}", "Channel", "OK", "Errors", "Retr");

		for (var i = 0; i < stats.Length; i++)
		{
			var stat = stats[i];

			Console.WriteLine("{0,8} {1,8} {2,8} {3,8}", i, stat.Ok, stat.HeaderError + stat.PayloadError, stat.Retransmitted);
		}
	}

	private static BluetoothAnalyzerRemoteControlPrx CreateBluetoothRemoteControlProxy(Ice.Communicator communicator)
	{
		var proxyString = $"{BluetoothAnalyzerRemoteControlIdentity.value}:tcp -h {HostIp} -p {HostPort}";
		var remoteControl = BluetoothAnalyzerRemoteControlPrxHelper.checkedCast(communicator.stringToProxy(proxyString));

		if(remoteControl == null)
		{
			throw new ApplicationException("Invalid proxy " + proxyString);
		}

		return remoteControl;
	}

	private static EthernetAnalyzerRemoteControlPrx CreateEthernetRemoteControlProxy(Ice.Communicator communicator)
	{
		var proxyString = $"{EthernetAnalyzerRemoteControlIdentity.value}:tcp -h {HostIp} -p {HostPort}";
		var remoteControl = EthernetAnalyzerRemoteControlPrxHelper.checkedCast(communicator.stringToProxy(proxyString));

		if(remoteControl == null)
		{
			throw new ApplicationException("Invalid proxy " + proxyString);
		}

		return remoteControl;
	}
}